"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const axios_1 = __importDefault(require("axios"));
const mqtt_topping_1 = require("@artcom/mqtt-topping");
const logger_1 = require("@artcom/logger");
async function retrieveBootstrapData(url, timeout, retryDelay, logger, debugBootstrapData) {
    if (debugBootstrapData) {
        logger.info("Using debug bootstrap data", { ...debugBootstrapData });
        return debugBootstrapData;
    }
    if (!url) {
        const bootstrapData = {
            tcpBrokerUri: process.env.TCP_BROKER_URI,
            httpBrokerUri: process.env.HTTP_BROKER_URI,
            configServerUri: process.env.CONFIG_SERVER_URI,
            device: process.env.DEVICE,
        };
        logger.info("Using bootstrap data from environment", { ...bootstrapData });
        return bootstrapData;
    }
    logger.info("Querying bootstrap data", { url });
    // eslint-disable-next-line no-constant-condition
    while (true) {
        try {
            const { data } = await axios_1.default.get(url, { timeout });
            logger.info("Bootstrap data received", { ...data });
            return data;
        }
        catch (error) {
            logger.error(`Query failed. Retrying in ${retryDelay}ms...`, { error: error.message });
            await delay(retryDelay);
        }
    }
}
function delay(time) {
    return new Promise((resolve) => setTimeout(resolve, time));
}
function connectMqttClient(serviceId, tcpBrokerUri, device, onParseError, logger) {
    const clientId = createClientId(serviceId, device);
    logger.info("Connecting to Broker", { tcpBrokerUri, clientId });
    const mqttClient = (0, mqtt_topping_1.connect)(tcpBrokerUri, { clientId, onParseError });
    mqttClient.on("connect", () => {
        logger.info("Connected to Broker");
    });
    mqttClient.on("close", () => {
        logger.error("Disconnected from Broker");
    });
    mqttClient.on("error", () => {
        logger.error("Error Connecting to Broker");
    });
    return mqttClient;
}
function createClientId(serviceId, device) {
    const uuid = Math.random().toString(16).substring(2, 10);
    if (device) {
        return `${serviceId}-${device}-${uuid}`;
    }
    else {
        return `${serviceId}-${uuid}`;
    }
}
function createQueryConfig(configServerUri) {
    return async (configPath, params = {}) => {
        const { version = "master", listFiles = false, includeCommitHash = false, parseJSON = true, } = params;
        const query = {
            url: `${configServerUri}/${version}/${configPath}?listFiles=${listFiles}`,
            transformResponse: parseJSON ? undefined : [],
        };
        return (0, axios_1.default)(query).then((response) => includeCommitHash
            ? { data: response.data, commitHash: response.headers["git-commit-hash"] }
            : response.data);
    };
}
module.exports = async function init(url, serviceId, { timeout = 2000, retryDelay = 10000, debugBootstrapData = undefined, onParseError = undefined, } = {}) {
    const logger = (0, logger_1.createLogger)();
    const data = await retrieveBootstrapData(url, timeout, retryDelay, logger, debugBootstrapData);
    return {
        logger,
        data,
        mqttClient: data.tcpBrokerUri
            ? connectMqttClient(serviceId, data.tcpBrokerUri, data.device, onParseError, logger)
            : undefined,
        httpClient: data.httpBrokerUri ? new mqtt_topping_1.HttpClient(data.httpBrokerUri) : undefined,
        queryConfig: data.configServerUri ? createQueryConfig(data.configServerUri) : undefined,
        unpublishRecursively: data.tcpBrokerUri && data.httpBrokerUri ? mqtt_topping_1.unpublishRecursively : undefined,
    };
};
//# sourceMappingURL=main.js.map